@@ -1,15 +1,14 @@
 import json
-import logging
-import sys
-
+import random
 import requests
 from requests.exceptions import HTTPError
+import sys
 import webbrowser
 
 from . import config, __version__
+from .classes.nodes import ChannelNode
 from .managers.progress import RestoreManager, Status
 from .managers.tree import ChannelManager
-from .sushi_bar_client import SushiBarClient
 
 # Fix to support Python 2.x.
 # http://stackoverflow.com/questions/954834/how-do-i-use-raw-input-in-python-3
@@ -18,20 +17,21 @@ try:
 except NameError:
     pass
 
-__logging_handler = None
 
 
 def uploadchannel_wrapper(chef, args, options):
     """
     Call `uploadchannel` with SushiBar monitoring and progress reporting enabled.
     Args:
-
+        args (dict): chef command line arguments
+        options (dict): extra key=value options given on the command line
     """
     try:
         args_and_options = args.copy()
         args_and_options.update(options)
         uploadchannel(chef, **args_and_options)
-        config.SUSHI_BAR_CLIENT.report_stage('COMPLETED', 0)
+        if config.SUSHI_BAR_CLIENT:
+            config.SUSHI_BAR_CLIENT.report_stage('COMPLETED', 0)
     except Exception as e:
         if config.SUSHI_BAR_CLIENT:
             config.SUSHI_BAR_CLIENT.report_stage('FAILURE', 0)
@@ -40,14 +40,13 @@ def uploadchannel_wrapper(chef, args, options):
     finally:
         if config.SUSHI_BAR_CLIENT:
             config.SUSHI_BAR_CLIENT.close()
-        config.LOGGER.removeHandler(__logging_handler)
 
 
-def uploadchannel(chef, verbose=False, update=False, thumbnails=False, download_attempts=3, resume=False, reset=False, step=Status.LAST.name, token="#", prompt=False, publish=False, warnings=False, compress=False, stage=False, **kwargs):
+def uploadchannel(chef, command='uploadchannel', update=False, thumbnails=False, download_attempts=3, resume=False, reset=False, step=Status.LAST.name, token="#", prompt=False, publish=False, compress=False, stage=False, **kwargs):
     """ uploadchannel: Upload channel to Kolibri Studio server
         Args:
             chef (BaseChef or subclass): class that implements the construct_channel method
-            verbose (bool): indicates whether to print process (optional)
+            command (str): the action we want to perform in this run
             update (bool): indicates whether to re-download files (optional)
             thumbnails (bool): indicates whether to automatically derive thumbnails from content (optional)
             download_attempts (int): number of times to retry downloading files (optional)
@@ -57,21 +56,13 @@ def uploadchannel(chef, verbose=False, update=False, thumbnails=False, download_
             token (str): content server authorization token
             prompt (bool): indicates whether to prompt user to open channel when done (optional)
             publish (bool): indicates whether to automatically publish channel (optional)
-            warnings (bool): indicates whether to print out warnings (optional)
             compress (bool): indicates whether to compress larger files (optional)
             stage (bool): indicates whether to stage rather than deploy channel (optional)
-            kwargs (dict): extra keyword args will be passed to get_channel and construct_channel (optional)
+            kwargs (dict): extra keyword args will be passed to construct_channel (optional)
         Returns: (str) link to access newly created channel
     """
 
     # Set configuration settings
-    global __logging_handler
-    level = logging.INFO if verbose else logging.WARNING if warnings else logging.ERROR
-    __logging_handler = logging.StreamHandler()
-    config.LOGGER.addHandler(__logging_handler)
-    logging.getLogger("requests").setLevel(logging.WARNING)
-    config.LOGGER.setLevel(level)
-
     config.UPDATE = update
     config.COMPRESS = compress
     config.THUMBNAILS = thumbnails
@@ -85,14 +76,14 @@ def uploadchannel(chef, verbose=False, update=False, thumbnails=False, download_
     # Get domain to upload to
     config.init_file_mapping_store()
 
-    # Authenticate user and check current Ricecooker version
-    username, token = authenticate_user(token)
-    check_version_number()
-
-    # Set dashboard client settings
-    config.LOGGER.info("Running get_channel... ")
-    channel = chef.get_channel(**kwargs)
-    config.SUSHI_BAR_CLIENT = SushiBarClient(channel, username, token)
+    if not command == 'dryrun':
+        # Authenticate user and check current Ricecooker version
+        username, token = authenticate_user(token)
+        config.LOGGER.info("Logged in with username {0}".format(username))
+        check_version_number()
+    else:
+        username = ''
+        token = ''
 
     config.LOGGER.info("\n\n***** Starting channel build process *****\n\n")
 
@@ -112,6 +103,8 @@ def uploadchannel(chef, verbose=False, update=False, thumbnails=False, download_
     if config.PROGRESS_MANAGER.get_status_val() <= Status.CONSTRUCT_CHANNEL.value:
         config.LOGGER.info("Calling construct_channel... ")
         channel = chef.construct_channel(**kwargs)
+        if 'sample' in kwargs and kwargs['sample']:
+            channel = select_sample_nodes(channel, size=kwargs['sample'])
         config.PROGRESS_MANAGER.set_channel(channel)
     channel = config.PROGRESS_MANAGER.channel
 
@@ -122,16 +115,20 @@ def uploadchannel(chef, verbose=False, update=False, thumbnails=False, download_
 
     # Download files if they haven't been downloaded already
     if config.PROGRESS_MANAGER.get_status_val() <= Status.DOWNLOAD_FILES.value:
-        print("Downloading files...")
+        config.LOGGER.info("Downloading files...")
         config.PROGRESS_MANAGER.set_files(*process_tree_files(tree))
 
+    if command == 'dryrun':
+        config.LOGGER.info('Command is dryrun so we are not uploading chanel.')
+        return
+
     # Set download manager in case steps were skipped
     files_to_diff = config.PROGRESS_MANAGER.files_downloaded
     config.FAILED_FILES = config.PROGRESS_MANAGER.files_failed
 
     # Get file diff if it hasn't been generated already
     if config.PROGRESS_MANAGER.get_status_val() <= Status.GET_FILE_DIFF.value:
-        print("Getting file diff...")
+        config.LOGGER.info("Getting file diff...")
         config.PROGRESS_MANAGER.set_diff(get_file_diff(tree, files_to_diff))
     file_diff = config.PROGRESS_MANAGER.file_diff
 
@@ -140,19 +137,19 @@ def uploadchannel(chef, verbose=False, update=False, thumbnails=False, download_
 
     # Upload files if they haven't been uploaded already
     if config.PROGRESS_MANAGER.get_status_val() <= Status.UPLOADING_FILES.value:
-        print("Uploading files...")
+        config.LOGGER.info("Uploading files...")
         config.PROGRESS_MANAGER.set_uploaded(upload_files(tree, file_diff))
 
     # Create channel on Kolibri Studio if it hasn't been created already
     if config.PROGRESS_MANAGER.get_status_val() <= Status.UPLOAD_CHANNEL.value:
-        print("Creating channel...")
+        config.LOGGER.info("Creating channel...")
         config.PROGRESS_MANAGER.set_channel_created(*create_tree(tree))
     channel_link = config.PROGRESS_MANAGER.channel_link
     channel_id = config.PROGRESS_MANAGER.channel_id
 
     # Publish tree if flag is set to True
     if config.PUBLISH and config.PROGRESS_MANAGER.get_status_val() <= Status.PUBLISH_CHANNEL.value:
-        print("Publishing channel...")
+        config.LOGGER.info("Publishing channel...")
         publish_tree(tree, channel_id)
         config.PROGRESS_MANAGER.set_published()
 
@@ -167,17 +164,22 @@ def uploadchannel(chef, verbose=False, update=False, thumbnails=False, download_
 
 def authenticate_user(token):
     """
-    Add the content curation Authorizatino `token` header to `config.SESSION`.
+    This function adds the studio Authorization `token` header to `config.SESSION`
+    and checks if the token is valid by performing a test call on the Studio API.
+    Args:
+        token (str): Studio authorization token
+    Returns:
+        username, token: Studio username and token if atthentication worked
     """
     config.SESSION.headers.update({"Authorization": "Token {0}".format(token)})
+    auth_endpoint = config.authentication_url()
     try:
-        response = config.SESSION.post(config.authentication_url())
+        response = config.SESSION.post(auth_endpoint)
         response.raise_for_status()
         user = json.loads(response._content.decode("utf-8"))
-        config.LOGGER.info("Logged in with username {0}".format(user['username']))
         return user['username'], token
     except HTTPError:
-        config.LOGGER.error("Invalid token: Credentials not found")
+        config.LOGGER.error("Studio token rejected by server " + auth_endpoint)
         sys.exit()
 
 def check_version_number():
@@ -237,7 +239,8 @@ def process_tree_files(tree):
     # Fill in values necessary for next steps
     config.LOGGER.info("Processing content...")
     files_to_diff = tree.process_tree(tree.channel)
-    config.SUSHI_BAR_CLIENT.report_statistics(files_to_diff)
+    if config.SUSHI_BAR_CLIENT:
+        config.SUSHI_BAR_CLIENT.report_statistics(files_to_diff, topic_count=tree.channel.get_topic_count())
     tree.check_for_files_failed()
     return files_to_diff, config.FAILED_FILES
 
@@ -275,8 +278,6 @@ def create_tree(tree):
     # Create tree
     config.LOGGER.info("\nCreating tree on Kolibri Studio...")
     channel_id, channel_link = tree.upload_tree()
-    # channel_id, channel_link = tree.upload_channel_structure()
-
     return channel_link, channel_id
 
 
@@ -289,3 +290,55 @@ def publish_tree(tree, channel_id):
     """
     config.LOGGER.info("\nPublishing tree to Kolibri... ")
     tree.publish(channel_id)
+
+
+def select_sample_nodes(channel, size=10, seed=42):
+    """
+    Build a sample tree of `size` leaf nodes from the channel `channel` to use
+    for debugging chef functionality without uploading the whole tree.
+    """
+    config.LOGGER.info('Selecting a sample of size ' + str(size))
+
+    # Step 1. channel to paths
+    node_paths = []   # list of tuples of the form (topic1, topic2, leafnode)
+    def walk_tree(parents_path, subtree):
+        for child in subtree.children:
+            child_path = parents_path + (child,)
+            if child.children:
+                # recurse
+                walk_tree(child_path, child)
+            else:
+                # emit leaf node
+                node_paths.append(child_path)
+    walk_tree((), channel)
+
+    # Step 2. sample paths
+    random.seed(seed)
+    sample_paths = random.sample(node_paths, size)
+    for node_path in sample_paths:
+        for node in node_path:
+            if node.children:
+                node.children = []  # empty children to clear old tree structure
+
+    # Step 3. paths to channel_sample
+    channel_sample = ChannelNode(
+        source_domain=channel.source_domain,
+        source_id=channel.source_id+'-sample',
+        title='Sample from ' + channel.title,
+        thumbnail=channel.thumbnail,
+        language=channel.language,
+        description='Sample from ' + channel.description
+    )
+    def attach(parent, node_path):
+        if len(node_path) == 1:
+            # leaf node
+            parent.add_child(node_path[0])
+        else:
+            child = node_path[0]
+            if not any(c.source_id == child.source_id for c in parent.children):
+                parent.add_child(child)
+            attach(child, node_path[1:])
+    for node_path in sample_paths:
+        attach(channel_sample, node_path)
+
+    return channel_sample
