@@ -1,11 +1,9 @@
 # Settings for rice cooker
-
-import os
-import json
-import logging
 import hashlib
+import logging.config
+import os
+
 import requests
-import logging
 from requests_file import FileAdapter
 
 UPDATE = False
@@ -15,13 +13,107 @@ PUBLISH = False
 PROGRESS_MANAGER = None
 SUSHI_BAR_CLIENT = None
 STAGE = False
+
+# Don't use this - call logging.getLogger(__name__) from each
+# individual module. Logging is configured centrally by calling
+# setup_logging()
 LOGGER = logging.getLogger()
 
-# Domain and file store location for uploading to production server
-DOMAIN = os.getenv('CONTENTWORKSHOP_URL', "https://contentworkshop.learningequality.org")
+
+# Keep error log when setup_logging is called
+_ERROR_LOG = None
+
+
+def setup_logging(level=logging.INFO, error_log=None, add_loggers=None):
+    """
+    Set up logging, useful to call from your sushi chef main script
+
+    :param level: Minimum default level for all loggers and handlers
+    :param error_log: Name of file to log (append) errors in
+    :param add_loggers: An iterable of other loggers to configure (['scrapy'])
+    """
+    global _ERROR_LOG
+
+    if not error_log:
+        error_log = _ERROR_LOG
+    else:
+        _ERROR_LOG = error_log
+
+    handlers = ["console"]
+    if error_log:
+        handlers.append("error")
+
+    # The default configuration of a logger (used in below config)
+    default_logger_config = {
+        "handlers": handlers,
+        "propagate": False,
+        "level": level,
+    }
+
+    config = {
+        'version': 1,
+        'disable_existing_loggers': False,
+        'formatters': {
+            'colored': {
+                '()': 'colorlog.ColoredFormatter',
+                'format': "%(log_color)s%(levelname)-8s%(reset)s %(blue)s%(message)s"
+            },
+            "simple_date": {
+                "format": "%(levelname)s %(asctime)s %(name)s %(message)s"
+            },
+        },
+        "handlers": {
+            "console": {
+                "level": level,
+                "class": "logging.StreamHandler",
+                "formatter": "colored",
+            },
+            "error": {
+                "level": logging.ERROR,
+                "class": "logging.FileHandler",
+                "filename": error_log or "void_error_log.log",
+                "formatter": "simple_date",
+            },
+        },
+        'loggers': {
+            '': {
+                "handlers": handlers,
+                "level": level,
+            },
+            'ricecooker': default_logger_config,
+        },
+    }
+
+    for logger in add_loggers or ():
+        config["loggers"][logger] = default_logger_config
+
+    logging.config.dictConfig(config)
+
+    # Silence noisy libraries loggers
+    logging.getLogger("requests").setLevel(logging.WARNING)
+    logging.getLogger("cachecontrol.controller").setLevel(logging.WARNING)
+    logging.getLogger("requests.packages").setLevel(logging.WARNING)
+    logging.getLogger("urllib3.util.retry").setLevel(logging.WARNING)
+    logging.getLogger("urllib3.connection").setLevel(logging.CRITICAL)
+    logging.getLogger("urllib3.connectionpool").setLevel(logging.WARNING)
+    logging.getLogger("PIL.PngImagePlugin").setLevel(logging.WARNING)
+
+
+# Setup default logging - can be called again to reconfigure
+setup_logging()
+
+
+# Domain and file store location for uploading to production Studio server
+DOMAIN_ENV = os.getenv('STUDIO_URL', None)
+if DOMAIN_ENV is None:  # check old ENV varable for backward compatibility
+    DOMAIN_ENV = os.getenv('CONTENTWORKSHOP_URL', None)
+DOMAIN = DOMAIN_ENV if DOMAIN_ENV else "https://api.studio.learningequality.org"
 if DOMAIN.endswith('/'):
     DOMAIN = DOMAIN.rstrip('/')
-FILE_STORE_LOCATION =  hashlib.md5(DOMAIN.encode('utf-8')).hexdigest()
+FILE_STORE_LOCATION = hashlib.md5(DOMAIN.encode('utf-8')).hexdigest()
+
+# Allow users to choose which phantomjs they use
+PHANTOMJS_PATH = os.getenv('PHANTOMJS_PATH', None)
 
 # URL for authenticating user on Kolibri Studio
 AUTHENTICATION_URL = "{domain}/api/internal/authenticate_user_internal"
@@ -35,15 +127,15 @@ FILE_DIFF_URL = "{domain}/api/internal/file_diff"
 # URL for uploading files to server
 FILE_UPLOAD_URL = "{domain}/api/internal/file_upload"
 
-# URL for uploading channel structure to server
-CHANNEL_STRUCTURE_UPLOAD_URL = "{domain}/api/internal/channel_structure_upload"
-
 # URL for creating channel on server
 CREATE_CHANNEL_URL = "{domain}/api/internal/create_channel"
 
 # URL for adding nodes to channel
 ADD_NODES_URL = "{domain}/api/internal/add_nodes"
 
+# URL for adding nodes to channel from file
+ADD_NODES_FROM_FILE_URL = "{domain}/api/internal/api_add_nodes_from_file"
+
 # URL for making final changes to channel
 FINISH_CHANNEL_URL = "{domain}/api/internal/finish_channel"
 
@@ -71,12 +163,16 @@ FAILED_FILES = []
 DOWNLOAD_SESSION = requests.Session()
 DOWNLOAD_SESSION.mount('file://', FileAdapter())
 
-# Sushi bar server
+# Environment variable indicating we should use a proxy for youtube_dl downloads
+USEPROXY = False
+USEPROXY = True if os.getenv('USEPROXY') is not None or os.getenv('PROXY_LIST') is not None else False
+
+# Sushibar server
 SUSHIBAR_URL = os.getenv('SUSHIBAR_URL', "https://sushibar.learningequality.org")
 if SUSHIBAR_URL.endswith('/'):
     SUSHIBAR_URL = SUSHIBAR_URL.rstrip('/')
 if not SUSHIBAR_URL.startswith('http'):
-   SUSHIBAR_URL = 'https://' + SUSHIBAR_URL        # in case only hostname given
+    SUSHIBAR_URL = 'https://' + SUSHIBAR_URL        # in case only hostname given
 SUSHI_BAR_HTTP = SUSHIBAR_URL
 SUSHI_BAR_WEBSOCKET = SUSHIBAR_URL.replace('http', 'ws', 1)
 SUSHI_BAR_CHANNEL_URL = "{domain}/api/channels/"
@@ -95,6 +191,8 @@ MAX_TITLE_LENGTH = 200
 MAX_SOURCE_ID_LENGTH = 200
 MAX_DESCRIPTION_LENGTH = 400
 MAX_AUTHOR_LENGTH = 200
+MAX_AGGREGATOR_LENGTH = 200
+MAX_PROVIDER_LENGTH = 200
 MAX_SOURCE_URL_LENGTH = 400
 MAX_ORIGINAL_FILENAME_LENGTH = 255
 MAX_LICENSE_DESCRIPTION_LENGTH = 400
@@ -118,7 +216,7 @@ MAX_CHAR_LIMITS = {
     },
     "author": {
         "kind": "Node",
-        "field": "source_id",
+        "field": "author",
         "max": MAX_AUTHOR_LENGTH
     },
     "question_source_url": {
@@ -146,6 +244,16 @@ MAX_CHAR_LIMITS = {
         "field": "copyright holder",
         "max": MAX_COPYRIGHT_HOLDER_LENGTH
     },
+    "provider": {
+        "kind": "Provider",
+        "field": "provider",
+        "max": MAX_PROVIDER_LENGTH
+    },
+    "aggregator": {
+        "kind": "Aggregator",
+        "field": "aggregator",
+        "max": MAX_AGGREGATOR_LENGTH
+    },
 }
 
 
@@ -160,9 +268,8 @@ def get_storage_path(filename):
     """
     directory = os.path.join(STORAGE_DIRECTORY, filename[0], filename[1])
     # Make storage directory for downloaded files if it doesn't already exist
-    if not os.path.exists(directory) :
+    if not os.path.exists(directory):
         os.makedirs(directory)
-
     return os.path.join(directory, filename)
 
 def authentication_url():
@@ -216,14 +323,6 @@ def file_upload_url():
     """
     return FILE_UPLOAD_URL.format(domain=DOMAIN)
 
-
-def channel_structure_upload_url():
-    """
-    Returns a string representation of the API endpoint for uploading a channel's structure.
-    """
-    return CHANNEL_STRUCTURE_UPLOAD_URL.format(domain=DOMAIN)
-
-
 def create_channel_url():
     """ create_channel_url: returns url to create channel
         Args: None
@@ -238,6 +337,13 @@ def add_nodes_url():
     """
     return ADD_NODES_URL.format(domain=DOMAIN)
 
+def add_nodes_from_file_url():
+    """ add_nodes_from_file_url: returns url to add nodes to channel using json file
+        Args: None
+        Returns: string url to add_nodes endpoint
+    """
+    return ADD_NODES_FROM_FILE_URL.format(domain=DOMAIN)
+
 def finish_channel_url():
     """ finish_channel_url: returns url to finish uploading a channel
         Args: None
