@@ -21,7 +21,7 @@ class ChannelManager:
             Args: None
             Returns: boolean indicating if tree is valid
         """
-        return self.channel.test_tree()
+        return self.channel.validate_tree()
 
     def process_tree(self, channel_node):
         """
@@ -32,7 +32,7 @@ class ChannelManager:
         """
         file_names = []
         self.process_tree_recur(file_names, channel_node)
-        return [x for x in set(file_names) if x]  # Remove any duplicate or null files
+        return [x for x in set(file_names) if x]  # Remove any duplicate or None filenames
 
     def process_tree_recur(self, file_names, node):
         """
@@ -56,14 +56,18 @@ class ChannelManager:
         if len(config.FAILED_FILES) > 0:
             config.LOGGER.error("   {} file(s) have failed to download".format(len(config.FAILED_FILES)))
             for f in config.FAILED_FILES:
-                title = "{0} {id}".format(f.node.kind.capitalize(), id=f.node.source_id)\
-                        if f.node else "{0} {id}".format("Question", id=f.assessment_item.source_id)
+                if f.node:              # files associated with a a content node
+                    info = "{0} {id}".format(f.node.kind.capitalize(), id=f.node.source_id)
+                elif f.assessment_item:  # files associated with an assessment item
+                    info = "{0} {id}".format("Question", id=f.assessment_item.source_id)
+                else:   # files not associated with a node or an assessment item
+                    info = f.__class__.__name__
                 file_identifier = f.__dict__
                 if hasattr(f, 'path') and f.path:
                     file_identifier = f.path
                 elif hasattr(f, 'youtube_url') and f.youtube_url:
                     file_identifier = f.youtube_url
-                config.LOGGER.warning("\t{0}: {id} \n\t   {err}".format(title, id=file_identifier, err=f.error))
+                config.LOGGER.warning("\t{0}: {id} \n\t   {err}".format(info, id=file_identifier, err=f.error))
         else:
             config.LOGGER.info("   All files were successfully downloaded")
 
@@ -118,33 +122,6 @@ class ChannelManager:
             self.failed_uploads = {}
             self.upload_files(current_fails)
 
-    def upload_channel_structure(self):
-        from datetime import datetime
-        start_time = datetime.now()
-        config.LOGGER.info('   Uploading structure of channel {0}'.format(self.channel.title))
-
-        channel_structure = {}
-        self.fill_channel_structure(channel_structure, self.channel, 0)
-        payload = {
-            'channel_id': self.channel.to_dict()['id'],
-            'channel_structure': channel_structure,
-            'stage': config.STAGE,
-        }
-        response = config.SESSION.post(config.channel_structure_upload_url(), data=json.dumps(payload, sort_keys=False))
-        response.raise_for_status()
-
-        new_channel = json.loads(response._content.decode('utf-8'))
-        end_time = datetime.now()
-        config.LOGGER.info("Upload time: {time}s".format(time=(end_time - start_time).total_seconds()))
-        return new_channel['channel_id'], config.open_channel_url(new_channel['channel_id'])
-
-    def fill_channel_structure(self, cur_dict, cur_node, sort_order):
-        children_dict = {}
-        child_sort_order = 0
-        for child in cur_node.children:
-            self.fill_channel_structure(children_dict, child)
-        cur_dict[cur_node.hashed_file_name] = children_dict
-
     def upload_tree(self):
         """ upload_tree: sends processed channel data to server to create tree
             Args: None
@@ -211,11 +188,16 @@ class ChannelManager:
             Returns: link to uploadedchannel
         """
         config.LOGGER.info("   Creating channel {0}".format(self.channel.title))
+        self.channel.truncate_fields()
         payload = {
             "channel_data":self.channel.to_dict(),
         }
         response = config.SESSION.post(config.create_channel_url(), data=json.dumps(payload))
-        response.raise_for_status()
+        try:
+            response.raise_for_status()
+        except Exception:
+            config.LOGGER.error("Error connecting to API: {}".format(response.text))
+            raise
         new_channel = json.loads(response._content.decode("utf-8"))
 
         return new_channel['root'], new_channel['channel_id']
@@ -261,6 +243,10 @@ class ChannelManager:
                     'root_id': root_id,
                     'content_data': payload_children
                 }
+
+                # When iceqube is integrated, use this method to utilize upload file optimizations
+                # response = config.SESSION.post(config.add_nodes_from_file_url(), files={'file': json.dumps(payload)})
+
                 response = config.SESSION.post(config.add_nodes_url(), data=json.dumps(payload))
                 if response.status_code != 200:
                     self.failed_node_builds[root_id] = {'node': current_node, 'error': response.reason}
